package com.htfp.weather.griddata.common;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author : shiyi
 * @Date : 2024/1/15 14:54
 * @Description : 文件变量名和表中变量名的映射
 */
public enum GfsVariableHeightEnum {
    //
    TEMP("temp", "Temperature_height_above_ground"),
    RH("humidity", "Relative_humidity_height_above_ground"),
    WIND_SPEED("windSpeed", "Wind_speed_height_above_ground"),
    WIND360("wind360", "Wind_direction_height_above_ground"),
    // GUST("gustSpeed", "Wind_speed_gust_surface"),
    PRECIP("precip", "Precipitation_rate_surface"),
    ;
    private static final ConcurrentHashMap<String, String> variableName = new ConcurrentHashMap<String, String>() {{
        put("temp", "Temperature_height_above_ground");
        put("humidity", "Relative_humidity_height_above_ground");
        put("windSpeed", "Wind_speed_height_above_ground");
        put("wind360", "Wind_direction_height_above_ground");
        put("precip", "Precipitation_rate_surface");
    }};
    String nameInApi;
    String nameInFile;

    GfsVariableHeightEnum(String nameInApi, String nameInFile) {
        this.nameInApi = nameInApi;
        this.nameInFile = nameInFile;
    }

    public String getNameInApi() {
        return nameInApi;
    }

    public String getNameInFile() {
        return nameInFile;
    }
    public static String getGfsVariableName(GfsVariableHeightEnum gfsVariableIsobaricEnum) {
        return variableName.get(gfsVariableIsobaricEnum.nameInApi);
    }
    public static String getGfsVariableName(String nameInApi) {
        return variableName.get(nameInApi);
    }

    public static String getVariableNameInApi(String nameInFile) {
        for (GfsVariableHeightEnum anEnum : values()) {
            if (anEnum.nameInFile.equals(nameInFile)) {
                return anEnum.nameInApi;
            }
        }
        return null;
    }

    public static List<String> getVariableNamesInApi() {
        return new ArrayList<>(variableName.keySet());
    }

    public static List<String> getVariableNamesInFile() {
        return new ArrayList<>(variableName.values());
    }
}
