package com.htfp.weather.griddata.common;

import com.google.gson.Gson;
import com.htfp.weather.download.GfsDataConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ucar.ma2.DataType;

import javax.annotation.PostConstruct;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author : shiyi
 * @Date : 2024/1/15 19:10
 * @Description : 表格meta配置
 */
@Slf4j @Component
public class TableConfigStatic {
    @Autowired
    GfsDataConfig dataConfig;
    /**
     * 表名和index名
     */
    public static String DATA_TABLE_NAME;
    public static String META_TABLE_NAME;
    public static String DATA_INDEX_NAME;
    public static String META_INDEX_NAME;

    /**
     * 数据属性：ID、本地数据文件名、目标变量名、变量维度大小、变量类型
     */
    public static String DATA_DIR;
    public static List<String> VARIABLE_LIST;
    public static int lonSize;
    public static List<Double> lonList;
    public static int latSize;
    public static List<Double> latList;
    public static int levSize;
    public static List<Integer> levList;
    public static int timeSizeMax = 72;
    public static List<String> timeList;
    public static DataType DATA_TYPE = DataType.FLOAT;

    // 数据过期时间，单位为秒
    public static int TIME_TO_LIVE;
    @PostConstruct
    private void initTableConfig() {
        initDatasetConfig();
        initLonList();
        initLatList();
        initLevList();
    }


    /**
     * 初始化表格配置
     */
    public static void initDatasetConfig() {
        log.info("init table config...");
        String path = System.getProperty("user.dir") + "./tableConf.json";
        try (InputStream f = new FileInputStream(path)){
            Gson gson = new Gson();
            Map map = gson.fromJson(IOUtils.toString(f), Map.class);
            DATA_TABLE_NAME = (String) map.get("dataTableName");
            META_TABLE_NAME = (String) map.get("metaTableName");
            DATA_INDEX_NAME = (String) map.get("dataIndexName");
            META_INDEX_NAME = (String) map.get("metaIndexName");
            DATA_DIR = (String) map.get("dataDir");
            VARIABLE_LIST = (ArrayList<String>) map.get("variableList");
            lonSize = ((Double) map.get("lonSize")).intValue();
            latSize = ((Double) map.get("latSize")).intValue();
            levSize = ((Double) map.get("levSize")).intValue();
            TIME_TO_LIVE = (int) (((Double) map.get("timeToLive")) * 24 *3600 ) ; // 配置文件的单位为天
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    private void initLonList() {
        double lonStart = dataConfig.getMinLon();
        double res = dataConfig.getResolution();
        lonList = new ArrayList<>();
        for (int i = 0; i < lonSize; i++) {
            lonList.add(lonStart + i * res);
        }
    }

    private void initLatList() {
        double latStart = dataConfig.getMinLat();
        double res = dataConfig.getResolution();
        latList = new ArrayList<>();
        for (int i = 0; i < latSize; i++) {
            latList.add(latStart + i * res);
        }
    }

    private void initLevList() {
        levList = Arrays.stream(dataConfig.getPressureLevels()).boxed().collect(Collectors.toList());
    }

}
