package com.htfp.weather.griddata.operation;

import com.alicloud.openservices.tablestore.model.TableOptions;
import com.alicloud.openservices.tablestore.model.search.FieldSchema;
import com.alicloud.openservices.tablestore.model.search.FieldType;
import com.alicloud.openservices.tablestore.model.search.IndexSchema;
import com.htfp.weather.griddata.common.TableConfigStatic;


import java.util.Arrays;

/**
 * @Author : shiyi
 * @Date : 2024/1/15 15:10
 * @Description : 创建表
 */
public class CreateTable extends BaseTableOperation {

    /**
     * we must create store before we can use it.
     * @throws Exception
     */
    private void createStore() throws Exception {
        TableOptions tableOptions = new TableOptions(TableConfigStatic.TIME_TO_LIVE, 1);
        this.tableStoreGrid.createStore(tableOptions);
    }

    /**
     * this example create an index which contains these columns: status, tag1, tag2, create_time.
     * you can create an index which contains any other columns.
     *
     * @throws Exception
     */
    private void createIndex() throws Exception {
        IndexSchema indexSchema = new IndexSchema();
        indexSchema.setFieldSchemas(Arrays.asList(
                new FieldSchema("status", FieldType.KEYWORD).setIndex(true).setEnableSortAndAgg(true),
                new FieldSchema("create_time", FieldType.LONG).setIndex(true).setEnableSortAndAgg(true),
                new FieldSchema("ref_time", FieldType.LONG).setIndex(true).setEnableSortAndAgg(true)
        ));
        this.tableStoreGrid.createMetaIndex(TableConfigStatic.META_INDEX_NAME, indexSchema);
    }

    public static void main(String[] args) throws Exception {
        CreateTable example = new CreateTable();
        try {
            example.createStore();
            example.createIndex();
        } finally {
            example.close();
        }
    }
}
